package org.fictus

import java.lang.reflect.{ Method => JMethod }
import org.fictus.internal._

abstract class Invoke(proxyName: String) extends ProxyStates {

    def interfaceInvoke(implicit context: MockContext) = new java.lang.reflect.InvocationHandler() {
        override def invoke(proxy: Any, method: JMethod, argsShit: Array[AnyRef]) = mockInvoke(proxy, method, argsShit)
    }

    def classInvoke(implicit context: MockContext) = new net.sf.cglib.proxy.InvocationHandler() {
        override def invoke(receiver: AnyRef, method: JMethod, argsShit: Array[AnyRef]): AnyRef = mockInvoke(receiver, method, argsShit)
    }

    private def mockInvoke(proxy: Any, method: JMethod, argsShit: Array[AnyRef])(implicit context: MockContext): AnyRef = {
        val methodName = method.getName
        methodName match {
            case "toString" => "Mocked " + proxyName
            case "equals"   => java.lang.Boolean.valueOf(true)
            case "hashCode" => java.lang.Integer.valueOf(1)
            case _ =>
                val args = Option(argsShit) match {
                    case Some(args) => args.toList
                    case None       => Nil
                }

                context.state.current match {
                    case Recording | DefaultRecording => record(Method(method.getName, method.getParameterTypes.toList, method.getReturnType, method.getExceptionTypes().toList), args)
                    case _                            => replay(Method(method.getName, method.getParameterTypes.toList, method.getReturnType, method.getExceptionTypes().toList), args)
                }
        }
    }
}